/*
Module : ScreenSaver.h
Purpose: Defines the interface for an MFC class framework to encapsulate screen savers
Created: PJN / 06-05-2000

Copyright (c) 2000 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/

//////////////////// Macros / Defines /////////////////////////////////////////

#ifndef __SCREENSAVER_H__
#define __SCREENSAVER_H__

#ifndef CSCREENSAVER_EXT_CLASS
#define CSCREENSAVER_EXT_CLASS
#endif //#ifndef CSCREENSAVER_EXT_CLASS

#ifndef CSCREENSAVER_EXT_API
#define CSCREENSAVER_EXT_API
#endif //#ifndef CSCREENSAVER_EXT_API


//////////////////// Classes //////////////////////////////////////////////////

class CSCREENSAVER_EXT_CLASS CScreenSaverCommandLineInfo : public CCommandLineInfo
{
public:
//Constructors / Destructors
  CScreenSaverCommandLineInfo() noexcept;
  CScreenSaverCommandLineInfo(const CScreenSaverCommandLineInfo&) = delete;
  CScreenSaverCommandLineInfo(CScreenSaverCommandLineInfo&&) = delete;
  ~CScreenSaverCommandLineInfo() = default; //NOLINT(modernize-use-override)

//Methods
  CScreenSaverCommandLineInfo& operator=(const CScreenSaverCommandLineInfo&) = delete;
  CScreenSaverCommandLineInfo& operator=(CScreenSaverCommandLineInfo&&) = delete;
  void ParseParam(const TCHAR* pszParam, BOOL bFlag, BOOL bLast) noexcept override;

//Enums
  enum class ShellCommand
  {
    ScreenSaverMode,
    PreviewMode,
    ShowConfigureDialog,
  };

//Member variables
  ShellCommand m_nShellCommand;
  HWND m_hParentWnd;
};


class CSCREENSAVER_EXT_CLASS CScreenSaverWnd : public CWnd
{
public:
//Constructors / Destructors
  CScreenSaverWnd() noexcept;
  CScreenSaverWnd(const CScreenSaverWnd&) = delete;
  CScreenSaverWnd(CScreenSaverWnd&&) = delete;
  ~CScreenSaverWnd() = default; //NOLINT(modernize-use-override)

//Methods
  CScreenSaverWnd& operator=(const CScreenSaverWnd&) = delete;
  CScreenSaverWnd& operator=(CScreenSaverWnd&&) = delete;
  virtual BOOL Create(_In_ bool bChild, _In_opt_ CWnd* pParentWnd); //NOLINT(clang-diagnostic-overloaded-virtual)
  void SetPreviewFlag(_In_ bool bPreview) noexcept { m_bPreview = bPreview; };

protected:
//Member Variables
  bool m_bGotCursorPos;
  CPoint m_LastPos;
  bool m_bPreview;

//Methods
  virtual bool GetBoundingRect(_In_ bool bChild, _In_opt_ CWnd* pParentWnd, _Inout_ CRect& rRect);
  virtual DWORD GetWindowStyle(_In_ bool bChild, _In_opt_ CWnd* pParentWnd);
  virtual DWORD GetExtendedWindowStyle(_In_ bool bChild, _In_opt_ CWnd* pParentWnd);
  void PostNcDestroy() override;

//Message handlers
  afx_msg void OnSysCommand(_In_ UINT nID, _In_ LPARAM lParam);
  afx_msg BOOL OnSetCursor(_In_opt_ CWnd* pWnd, _In_ UINT nHitTest, _In_ UINT message);
  afx_msg void OnActivate(_In_ UINT nState, _In_opt_ CWnd* pWndOther, _In_ BOOL bMinimized);
  afx_msg void OnActivateApp(_In_ BOOL bActive, _In_ DWORD dwThreadID);
  afx_msg void OnMouseMove(_In_ UINT nFlags, _In_ CPoint point);
  afx_msg void OnLButtonDown(_In_ UINT nFlags, _In_ CPoint point);
  afx_msg void OnRButtonDown(_In_ UINT nFlags, _In_ CPoint point);
  afx_msg void OnMButtonDown(_In_ UINT nFlags, _In_ CPoint point);
  afx_msg void OnKeyDown(_In_ UINT nChar, _In_ UINT nRepCnt, _In_ UINT nFlags);
  afx_msg void OnSysKeyDown(_In_ UINT nChar, _In_ UINT nRepCnt, _In_ UINT nFlags);
  afx_msg void OnPaint();

  DECLARE_DYNCREATE(CScreenSaverWnd)
  DECLARE_MESSAGE_MAP()
};


class CSCREENSAVER_EXT_CLASS CScreenSaverApp : public CWinApp
{
public:
//Constructors / Destructors
  CScreenSaverApp() noexcept;
  CScreenSaverApp(const CScreenSaverApp&) = delete;
  CScreenSaverApp(CScreenSaverApp&&) = delete;
  ~CScreenSaverApp() = default; //NOLINT(modernize-use-override)

//Methods
  CScreenSaverApp& operator=(const CScreenSaverApp&) = delete;
  CScreenSaverApp& operator=(CScreenSaverApp&&) = delete;

protected:
//Member variables
  CRuntimeClass* m_pRuntimeScreenSaverWnd;

//Methods
  void SetScreenSaverTemplate(_In_ CRuntimeClass* pRuntimeScreenSaverWnd);
  virtual void ParseCommandLine(_In_ CScreenSaverCommandLineInfo& rCmdInfo) noexcept;
  virtual bool ProcessShellCommand(_In_ CScreenSaverCommandLineInfo& rCmdInfo);
  virtual void ShowConfigureDialog(_In_opt_ CWnd* pParentWnd);
  BOOL InitInstance() override;

  DECLARE_MESSAGE_MAP()
};


#endif //#ifndef __SCREENSAVER_H__
